@@ -8,19 +8,19 @@
 #' @seealso [fortify.lm()]
 #' @param model model or other R object to convert to data frame
 #' @param data original dataset, if needed
-#' @param ... other arguments passed to methods
+#' @inheritParams rlang::args_dots_used
 #' @export
-fortify <- function(model, data, ...) UseMethod("fortify")
+fortify <- function(model, data, ...) {
+  warn_dots_used()
+  UseMethod("fortify")
+}
 
 #' @export
 fortify.data.frame <- function(model, data, ...) model
 #' @export
 fortify.tbl_df <- function(model, data, ...) model
 #' @export
-fortify.tbl <- function(model, data, ...) {
-  check_installed("dplyr", reason = "to work with `tbl` objects.")
-  dplyr::collect(model)
-}
+fortify.tbl <- function(model, data, ...) as.data.frame(model)
 #' @export
 fortify.NULL <- function(model, data, ...) waiver()
 #' @export
@@ -44,54 +44,66 @@ fortify.grouped_df <- function(model, data, ...) {
 # There are a lot of ways that dim(), colnames(), or as.data.frame() could
 # do non-sensical things (they are not even guaranteed to work!) hence the
 # paranoid mode.
-.prevalidate_data_frame_like_object <- function(data) {
+check_data_frame_like <- function(data) {
   orig_dims <- dim(data)
-  if (!vec_is(orig_dims, integer(), size=2))
-    cli::cli_abort(paste0("{.code dim(data)} must return ",
-                          "an {.cls integer} of length 2."))
-  if (anyNA(orig_dims) || any(orig_dims < 0))  # extra-paranoid mode
-    cli::cli_abort(paste0("{.code dim(data)} can't have {.code NA}s ",
-                          "or negative values."))
+  if (!vec_is(orig_dims, integer(), size = 2)) {
+    cli::cli_abort(
+      "{.code dim(data)} must return an {.cls integer} of length 2."
+    )
+  }
+  if (anyNA(orig_dims) || any(orig_dims < 0)) { # extra-paranoid mode
+    cli::cli_abort(
+      "{.code dim(data)} can't have {.code NA}s or negative values."
+    )
+  }
   orig_colnames <- colnames(data)
-  if (!vec_is(orig_colnames, character(), size = ncol(data)))
-    cli::cli_abort(paste0("{.code colnames(data)} must return a ",
-                          "{.cls character} of length {.code ncol(data)}."))
+  if (!vec_is(orig_colnames, character(), size = ncol(data))) {
+    cli::cli_abort(
+      "{.code colnames(data)} must return a {.cls character} of length {.code ncol(data)}."
+    )
+  }
+  invisible()
 }
-.postvalidate_data_frame_like_object <- function(df, data) {
+check_data_frame_conversion <- function(new, old) {
   msg0 <- "{.code as.data.frame(data)} must "
-  if (!is.data.frame(df))
+  if (!is.data.frame(new)) {
     cli::cli_abort(paste0(msg0, "return a {.cls data.frame}."))
-  if (!identical(dim(df), dim(data)))
+  }
+  if (!identical(dim(new), dim(old))) {
     cli::cli_abort(paste0(msg0, "preserve dimensions."))
-  if (!identical(colnames(df), colnames(data)))
+  }
+  if (!identical(colnames(new), colnames(old))) {
     cli::cli_abort(paste0(msg0, "preserve column names."))
+  }
+  invisible()
 }
 validate_as_data_frame <- function(data) {
-  if (is.data.frame(data))
+  if (is.data.frame(data)) {
     return(data)
-  .prevalidate_data_frame_like_object(data)
+  }
+  check_data_frame_like(data)
   df <- as.data.frame(data)
-  .postvalidate_data_frame_like_object(df, data)
+  check_data_frame_conversion(df, data)
   df
 }
 
 #' @export
 fortify.default <- function(model, data, ...) {
-  msg0 <- paste0(
-    "{{.arg data}} must be a {{.cls data.frame}}, ",
-    "or an object coercible by {{.fn fortify}}, or a valid ",
-    "{{.cls data.frame}}-like object coercible by {{.fn as.data.frame}}"
+  msg <- paste0(
+    "{.arg data} must be a {.cls data.frame}, ",
+    "or an object coercible by {.fn fortify}, or a valid ",
+    "{.cls data.frame}-like object coercible by {.fn as.data.frame}"
   )
-  if (inherits(model, "uneval")) {
+  if (is_mapping(model)) {
     msg <- c(
-      glue(msg0, ", not {obj_type_friendly(model)}."),
+      paste0(msg, ", not ", obj_type_friendly(model), "."),
       "i" = "Did you accidentally pass {.fn aes} to the {.arg data} argument?"
     )
     cli::cli_abort(msg)
   }
-  msg0 <- paste0(msg0, ". ")
+  msg <- paste0(msg, ".")
   try_fetch(
     validate_as_data_frame(model),
-    error = function(cnd) cli::cli_abort(glue(msg0), parent = cnd)
+    error = function(cnd) cli::cli_abort(msg, parent = cnd)
   )
 }
