@@ -1,3 +1,65 @@
+#' @rdname Geom
+#' @format NULL
+#' @usage NULL
+#' @export
+GeomSmooth <- ggproto(
+  "GeomSmooth", Geom,
+  setup_params = function(data, params) {
+    params$flipped_aes <- has_flipped_aes(data, params, range_is_orthogonal = TRUE, ambiguous = TRUE)
+    params$se <- params$se %||%
+      if (params$flipped_aes) {
+        all(c("xmin", "xmax") %in% names(data))
+      } else {
+        all(c("ymin", "ymax") %in% names(data))
+      }
+
+    params
+  },
+
+  extra_params = c("na.rm", "orientation"),
+
+  setup_data = function(data, params) {
+    GeomLine$setup_data(data, params)
+  },
+
+  # The `se` argument is set to false here to make sure drawing the
+  # geom and drawing the legend is in synch. If the geom is used by a
+  # stat that doesn't set the `se` argument then `se` will be missing
+  # and the legend key won't be drawn. With `se = FALSE` here the
+  # ribbon won't be drawn either in that case, keeping the overall
+  # behavior predictable and sensible. The user will realize that they
+  # need to set `se = TRUE` to obtain the ribbon and the legend key.
+  draw_group = function(data, panel_params, coord, lineend = "butt", linejoin = "round",
+                        linemitre = 10, se = FALSE, flipped_aes = FALSE) {
+    ribbon <- transform(data, colour = NA)
+    path <- transform(data, alpha = NA)
+
+    ymin <- flipped_names(flipped_aes)$ymin
+    ymax <- flipped_names(flipped_aes)$ymax
+    has_ribbon <- se && !is.null(data[[ymax]]) && !is.null(data[[ymin]])
+
+    gList(
+      if (has_ribbon) GeomRibbon$draw_group(ribbon, panel_params, coord, flipped_aes = flipped_aes),
+      GeomLine$draw_panel(path, panel_params, coord, lineend = lineend, linejoin = linejoin, linemitre = linemitre)
+    )
+  },
+
+  draw_key = draw_key_smooth,
+
+  required_aes = c("x", "y"),
+  optional_aes = c("ymin", "ymax"),
+
+  default_aes = aes(
+    colour = from_theme(colour %||% accent),
+    fill = from_theme(fill %||% col_mix(ink, paper, 0.6)),
+    linewidth = from_theme(2 * linewidth),
+    linetype = from_theme(linetype),
+    weight = 1, alpha = 0.4
+  ),
+
+  rename_size = TRUE
+)
+
 #' Smoothed conditional means
 #'
 #' Aids the eye in seeing patterns in the presence of overplotting.
@@ -9,12 +71,12 @@
 #' `predictdf()` generic and its methods.  For most methods the standard
 #' error bounds are computed using the [predict()] method -- the
 #' exceptions are `loess()`, which uses a t-based approximation, and
-#' `glm()`, where the normal confidence interval is constructed on the link
+#' `glm()`, where the normal confidence band is constructed on the link
 #' scale and then back-transformed to the response scale.
 #'
 #' @eval rd_orientation()
 #'
-#' @eval rd_aesthetics("geom", "smooth")
+#' @aesthetics GeomSmooth
 #' @inheritParams layer
 #' @inheritParams geom_bar
 #' @param geom,stat Use to override the default connection between
@@ -102,7 +164,7 @@ geom_smooth <- function(mapping = NULL, data = NULL,
     ...
   )
   if (identical(stat, "smooth")) {
-    params$method <- method
+    params[["method"]] <- method
     params$formula <- formula
   }
 
@@ -117,52 +179,3 @@ geom_smooth <- function(mapping = NULL, data = NULL,
     params = params
   )
 }
-
-#' @rdname ggplot2-ggproto
-#' @format NULL
-#' @usage NULL
-#' @export
-GeomSmooth <- ggproto("GeomSmooth", Geom,
-  setup_params = function(data, params) {
-    params$flipped_aes <- has_flipped_aes(data, params, range_is_orthogonal = TRUE, ambiguous = TRUE)
-    params
-  },
-
-  extra_params = c("na.rm", "orientation"),
-
-  setup_data = function(data, params) {
-    GeomLine$setup_data(data, params)
-  },
-
-  # The `se` argument is set to false here to make sure drawing the
-  # geom and drawing the legend is in synch. If the geom is used by a
-  # stat that doesn't set the `se` argument then `se` will be missing
-  # and the legend key won't be drawn. With `se = FALSE` here the
-  # ribbon won't be drawn either in that case, keeping the overall
-  # behavior predictable and sensible. The user will realize that they
-  # need to set `se = TRUE` to obtain the ribbon and the legend key.
-  draw_group = function(data, panel_params, coord, lineend = "butt", linejoin = "round",
-                        linemitre = 10, se = FALSE, flipped_aes = FALSE) {
-    ribbon <- transform(data, colour = NA)
-    path <- transform(data, alpha = NA)
-
-    ymin = flipped_names(flipped_aes)$ymin
-    ymax = flipped_names(flipped_aes)$ymax
-    has_ribbon <- se && !is.null(data[[ymax]]) && !is.null(data[[ymin]])
-
-    gList(
-      if (has_ribbon) GeomRibbon$draw_group(ribbon, panel_params, coord, flipped_aes = flipped_aes),
-      GeomLine$draw_panel(path, panel_params, coord, lineend = lineend, linejoin = linejoin, linemitre = linemitre)
-    )
-  },
-
-  draw_key = draw_key_smooth,
-
-  required_aes = c("x", "y"),
-  optional_aes = c("ymin", "ymax"),
-
-  default_aes = aes(colour = "#3366FF", fill = "grey60", linewidth = 1,
-    linetype = 1, weight = 1, alpha = 0.4),
-
-  rename_size = TRUE
-)
