@@ -83,6 +83,9 @@
 #' # Interpreting the labels as plotmath expressions
 #' p + facet_grid(. ~ cyl2)
 #' p + facet_grid(. ~ cyl2, labeller = label_parsed)
+#'
+#' # Include optional argument in label function
+#' p + facet_grid(. ~ cyl, labeller = \(x) label_both(x, sep = "="))
 #' }
 #' @name labellers
 NULL
@@ -111,21 +114,17 @@ label_value <- function(labels, multi_line = TRUE) {
 # currently needed for Roxygen
 class(label_value) <- c("function", "labeller")
 
-# Helper for label_both
-label_variable <- function(labels, multi_line = TRUE) {
-  if (multi_line) {
-    row <- as.list(names(labels))
-  } else {
-    row <- list(paste(names(labels), collapse = ", "))
-  }
-  lapply(row, rep, nrow(labels) %||% length(labels[[1]]))
-}
-
 #' @rdname labellers
 #' @export
 label_both <- function(labels, multi_line = TRUE, sep = ": ") {
   value <- label_value(labels, multi_line = multi_line)
-  variable <- label_variable(labels, multi_line = multi_line)
+
+  if (isTRUE(multi_line)) {
+    row <- as.list(names(labels))
+  } else {
+    row <- list(paste(names(labels), collapse = ", "))
+  }
+  variable <- lapply(row, rep, nrow(labels) %||% length(labels[[1]]))
 
   if (multi_line) {
     out <- vector("list", length(value))
@@ -173,14 +172,6 @@ label_parsed <- function(labels, multi_line = TRUE) {
 }
 class(label_parsed) <- c("function", "labeller")
 
-find_names <- function(expr) {
-  if (is.call(expr)) {
-    unlist(lapply(expr[-1], find_names))
-  } else if (is.name(expr)) {
-    as.character(expr)
-  }
-}
-
 #' Label with mathematical expressions
 #'
 #' `label_bquote()` offers a flexible way of labelling
@@ -305,7 +296,7 @@ as_labeller <- function(x, default = label_value, multi_line = TRUE) {
       x(labels)
     } else if (is.function(x)) {
       default(lapply(labels, x))
-    } else if (is.formula(x)) {
+    } else if (is_formula(x)) {
       default(lapply(labels, as_function(x)))
     } else if (is.character(x)) {
       default(lapply(labels, function(label) x[label]))
@@ -320,7 +311,7 @@ as_labeller <- function(x, default = label_value, multi_line = TRUE) {
 #'
 #' This function makes it easy to assign different labellers to
 #' different factors. The labeller can be a function or it can be a
-#' named character vectors that will serve as a lookup table.
+#' named character vector that will serve as a lookup table.
 #'
 #' In case of functions, if the labeller has class `labeller`, it
 #' is directly applied on the data frame of labels. Otherwise, it is
@@ -420,7 +411,7 @@ labeller <- function(..., .rows = NULL, .cols = NULL,
                      keep.as.numeric = deprecated(), .multi_line = TRUE,
                      .default = label_value) {
   if (lifecycle::is_present(keep.as.numeric)) {
-    deprecate_warn0("2.0.0", "labeller(keep.as.numeric)")
+    lifecycle::deprecate_stop("2.0.0", "labeller(keep.as.numeric)")
   }
   dots <- list2(...)
   .default <- as_labeller(.default)
@@ -547,7 +538,7 @@ build_strip <- function(label_df, labeller, theme, horizontal) {
 #' Takes the output from title_spec, adds margins, creates gList with strip
 #' background and label, and returns gtable matrix.
 #'
-#' @param grobs Output from [titleGrob()].
+#' @param grobs Output from `titleGrob()`.
 #' @param theme Theme object.
 #' @param horizontal Whether the strips are horizontal (e.g. x facets) or not.
 #' @param clip should drawing be clipped to the specified cells (‘"on"’),the
@@ -555,7 +546,7 @@ build_strip <- function(label_df, labeller, theme, horizontal) {
 #'
 #' @noRd
 assemble_strips <- function(grobs, theme, horizontal = TRUE, clip) {
-  if (length(grobs) == 0 || is.zero(grobs[[1]])) {
+  if (length(grobs) == 0 || is_zero(grobs[[1]])) {
     # Subsets matrix of zeroGrobs to correct length (#4050)
     grobs <- grobs[seq_len(NROW(grobs))]
     return(grobs)
@@ -586,22 +577,18 @@ assemble_strips <- function(grobs, theme, horizontal = TRUE, clip) {
   })
 }
 
-# Check for old school labeller
-check_labeller <- function(labeller) {
+# Reject old school labeller
+validate_labeller <- function(labeller) {
+
   labeller <- match.fun(labeller)
   is_deprecated <- all(c("variable", "value") %in% names(formals(labeller)))
 
-  if (is_deprecated) {
-    old_labeller <- labeller
-    labeller <- function(labels) {
-      Map(old_labeller, names(labels), labels)
-    }
-    # TODO Update to lifecycle after next lifecycle release
-    cli::cli_warn(c(
-      "The {.arg labeller} API has been updated. Labellers taking {.arg variable} and {.arg value} arguments are now deprecated.",
-      "i" = "See labellers documentation."
-    ))
+  if (!is_deprecated) {
+    return(labeller)
   }
 
-  labeller
+  lifecycle::deprecate_stop(
+    "2.0.0",
+    what = I("Providing a labeller with `variable` and `value` arguments")
+  )
 }
