@@ -38,53 +38,39 @@ layer_sf <- function(geom = NULL, stat = NULL,
 LayerSf <- ggproto("LayerSf", Layer,
   legend_key_type = NULL,
 
-  # This field carry state throughout rendering but will always be
-  # calculated before use
-  computed_legend_key_type = NULL,
-
   setup_layer = function(self, data, plot) {
     # process generic layer setup first
     data <- ggproto_parent(Layer, self)$setup_layer(data, plot)
 
     # automatically determine the name of the geometry column
     # and add the mapping if it doesn't exist
-    if ((isTRUE(self$inherit.aes) && is.null(self$computed_mapping$geometry) &&
-         is.null(plot$computed_mapping$geometry)) ||
-        (!isTRUE(self$inherit.aes) && is.null(self$computed_mapping$geometry))) {
-      if (is_sf(data)) {
-        geometry_col <- attr(data, "sf_column")
-        self$computed_mapping$geometry <- sym(geometry_col)
-      }
-    }
-
-    # automatically determine the legend type
-    if (is.null(self$legend_key_type)) {
-      # first, set default value in case downstream tests fail
-      self$computed_legend_key_type <- "polygon"
-
-      # now check if the type should not be polygon
-      if (!is.null(self$computed_mapping$geometry) && quo_is_symbol(self$computed_mapping$geometry)) {
-        geometry_column <- as_name(self$computed_mapping$geometry)
-        if (inherits(data[[geometry_column]], "sfc")) {
-          sf_type <- detect_sf_type(data[[geometry_column]])
-          if (sf_type == "point") {
-            self$computed_legend_key_type <- "point"
-          } else if (sf_type == "line") {
-            self$computed_legend_key_type <- "line"
-          }
-        }
-      }
-    } else {
-      self$computed_legend_key_type <- self$legend_key_type
+    if (is.null(self$computed_mapping$geometry) && is_sf(data)) {
+      geometry_col <- attr(data, "sf_column")
+      self$computed_mapping$geometry <- sym(geometry_col)
     }
     data
   },
   compute_geom_1 = function(self, data) {
     data <- ggproto_parent(Layer, self)$compute_geom_1(data)
 
+    # Determine the legend type
+    legend_type <- self$legend_key_type
+    if (is.null(legend_type)) {
+      legend_type <- switch(
+        detect_sf_type(data$geometry),
+        point = "point", line = "line", "other"
+      )
+    }
+
     # Add legend type after computed_geom_params has been calculated
-    self$computed_geom_params$legend <- self$computed_legend_key_type
+    self$computed_geom_params$legend <- legend_type
     data
+  },
+
+  compute_geom_2 = function(self, data, params = self$aes_params, ...) {
+    if (empty(data)) return(data)
+    data$geometry <- data$geometry %||% self$computed_geom_params$legend
+    ggproto_parent(Layer, self)$compute_geom_2(data, params, ...)
   }
 )
 
@@ -92,7 +78,12 @@ LayerSf <- ggproto("LayerSf", Layer,
 geom_column <- function(data) {
   w <- which(vapply(data, inherits, TRUE, what = "sfc"))
   if (length(w) == 0) {
-    "geometry" # avoids breaks when objects without geometry list-column are examined
+    if (!is.character(data[["geometry"]])) {
+      "geometry" # avoids breaks when objects without geometry list-column are examined
+    } else {
+      # Avoids a rare case where computed_geom_params$legend is present but there is no actual geometry column
+      ""
+    }
   } else {
     # this may not be best in case more than one geometry list-column is present:
     if (length(w) > 1)
@@ -113,6 +104,9 @@ scale_type.sfc <- function(x) "identity"
 
 # helper function to determine the geometry type of sf object
 detect_sf_type <- function(sf) {
+  if (is.null(sf)) {
+    return("other")
+  }
   geometry_type <- unique0(as.character(sf::st_geometry_type(sf)))
   if (length(geometry_type) != 1)  geometry_type <- "GEOMETRY"
   sf_types[geometry_type]
