@@ -2,7 +2,7 @@
 #'
 #' `scale_linewidth` scales the width of lines and polygon strokes. Due to
 #' historical reasons, it is also possible to control this with the `size`
-#' aesthetic, but using `linewidth` is encourage to clearly differentiate area
+#' aesthetic, but using `linewidth` is encouraged to clearly differentiate area
 #' aesthetics from stroke width aesthetics.
 #'
 #' @name scale_linewidth
@@ -31,12 +31,17 @@ NULL
 #' @usage NULL
 scale_linewidth_continuous <- function(name = waiver(), breaks = waiver(),
                                        labels = waiver(), limits = NULL,
-                                       range = c(1, 6), transform = "identity",
+                                       range = NULL, transform = "identity",
                                        trans = deprecated(),
-                                       guide = "legend") {
-  continuous_scale("linewidth", palette = pal_rescale(range), name = name,
-                   breaks = breaks, labels = labels, limits = limits,
-                   transform = transform, trans = trans, guide = guide)
+                                       guide = "legend",
+                                       aesthetics = "linewidth") {
+  palette <- if (!is.null(range)) pal_rescale(range) else NULL
+  continuous_scale(
+    aesthetics, palette = palette, name = name,
+    breaks = breaks, labels = labels, limits = limits,
+    transform = transform, trans = trans, guide = guide,
+    fallback.palette = pal_rescale(c(1, 6))
+  )
 }
 
 #' @rdname scale_linewidth
@@ -46,13 +51,17 @@ scale_linewidth <- scale_linewidth_continuous
 #' @rdname scale_linewidth
 #' @export
 scale_linewidth_binned <- function(name = waiver(), breaks = waiver(), labels = waiver(),
-                              limits = NULL, range = c(1, 6), n.breaks = NULL,
+                              limits = NULL, range = NULL, n.breaks = NULL,
                               nice.breaks = TRUE, transform = "identity",
-                              trans = deprecated(), guide = "bins") {
-  binned_scale("linewidth", palette = pal_rescale(range), name = name,
-               breaks = breaks, labels = labels, limits = limits,
-               transform = transform, trans = trans, n.breaks = n.breaks,
-               nice.breaks = nice.breaks, guide = guide)
+                              trans = deprecated(), guide = "bins", aesthetics = "linewidth") {
+  palette <- if (!is.null(range)) pal_rescale(range) else NULL
+  binned_scale(
+    aesthetics, palette = palette, name = name,
+    breaks = breaks, labels = labels, limits = limits,
+    transform = transform, trans = trans, n.breaks = n.breaks,
+    nice.breaks = nice.breaks, guide = guide,
+    fallback.palette = pal_rescale(c(1, 6))
+  )
 }
 
 #' @rdname scale_linewidth
@@ -68,12 +77,15 @@ scale_linewidth_discrete <- function(...) {
 #' @rdname scale_linewidth
 #' @export
 #' @usage NULL
-scale_linewidth_ordinal <- function(name = waiver(), ..., range = c(2, 6)) {
-  force(range)
-
+scale_linewidth_ordinal <- function(name = waiver(), ..., range = NULL, aesthetics = "linewidth") {
+  palette <- if (!is.null(range)) {
+    function(n) seq(range[1], range[2], length.out = n)
+  } else {
+    NULL
+  }
   discrete_scale(
-    "linewidth", name = name,
-    palette = function(n) seq(range[1], range[2], length.out = n),
+    aesthetics, name = name, palette = palette,
+    fallback.palette = function(n) seq(2, 6, length.out = n),
     ...
   )
 }
@@ -81,19 +93,23 @@ scale_linewidth_ordinal <- function(name = waiver(), ..., range = c(2, 6)) {
 #' @rdname scale_linewidth
 #' @export
 #' @usage NULL
-scale_linewidth_datetime <- function(name = waiver(), ..., range = c(1, 6)) {
+scale_linewidth_datetime <- function(name = waiver(), ..., range = NULL, aesthetics = "linewidth") {
+  palette <- if (!is.null(range)) pal_rescale(range) else NULL
   datetime_scale(
-    "linewidth", transform = "time", name = name,
-    palette = pal_rescale(range), ...
+    aesthetics, transform = "time", name = name,
+    palette = palette, fallback.palette = pal_rescale(c(1, 6)),
+    ...
   )
 }
 
 #' @rdname scale_linewidth
 #' @export
 #' @usage NULL
-scale_linewidth_date <- function(name = waiver(), ..., range = c(1, 6)) {
+scale_linewidth_date <- function(name = waiver(), ..., range = NULL, aesthetics = "linewidth") {
+  palette <- if (!is.null(range)) pal_rescale(range) else NULL
   datetime_scale(
-    "linewidth", transform = "date", name = name,
-    palette = pal_rescale(range), ...
+    aesthetics, transform = "date", name = name,
+    palette = palette, fallback.palette = pal_rescale(c(1, 6)),
+    ...
   )
 }
