@@ -1,76 +1,9 @@
-#' @param bw The smoothing bandwidth to be used.
-#'   If numeric, the standard deviation of the smoothing kernel.
-#'   If character, a rule to choose the bandwidth, as listed in
-#'   [stats::bw.nrd()]. Note that automatic calculation of the bandwidth does
-#'   not take weights into account.
-#' @param adjust A multiplicate bandwidth adjustment. This makes it possible
-#'    to adjust the bandwidth while still using the a bandwidth estimator.
-#'    For example, `adjust = 1/2` means use half of the default bandwidth.
-#' @param kernel Kernel. See list of available kernels in [density()].
-#' @param n number of equally spaced points at which the density is to be
-#'   estimated, should be a power of two, see [density()] for
-#'   details
-#' @param trim If `FALSE`, the default, each density is computed on the
-#'   full range of the data. If `TRUE`, each density is computed over the
-#'   range of that group: this typically means the estimated x values will
-#'   not line-up, and hence you won't be able to stack density values.
-#'   This parameter only matters if you are displaying multiple densities in
-#'   one plot or if you are manually adjusting the scale limits.
-#' @param bounds Known lower and upper bounds for estimated data. Default
-#'   `c(-Inf, Inf)` means that there are no (finite) bounds. If any bound is
-#'   finite, boundary effect of default density estimation will be corrected by
-#'   reflecting tails outside `bounds` around their closest edge. Data points
-#'   outside of bounds are removed with a warning.
-#' @eval rd_computed_vars(
-#'  density  = "density estimate.",
-#'  count    = "density * number of points - useful for stacked density plots.",
-#'  scaled   = "density estimate, scaled to maximum of 1.",
-#'  n        = "number of points.",
-#'  ndensity = "alias for `scaled`, to mirror the syntax of [`stat_bin()`]."
-#' )
-#' @export
-#' @rdname geom_density
-stat_density <- function(mapping = NULL, data = NULL,
-                         geom = "area", position = "stack",
-                         ...,
-                         bw = "nrd0",
-                         adjust = 1,
-                         kernel = "gaussian",
-                         n = 512,
-                         trim = FALSE,
-                         na.rm = FALSE,
-                         bounds = c(-Inf, Inf),
-                         orientation = NA,
-                         show.legend = NA,
-                         inherit.aes = TRUE) {
-
-  layer(
-    data = data,
-    mapping = mapping,
-    stat = StatDensity,
-    geom = geom,
-    position = position,
-    show.legend = show.legend,
-    inherit.aes = inherit.aes,
-    params = list2(
-      bw = bw,
-      adjust = adjust,
-      kernel = kernel,
-      n = n,
-      trim = trim,
-      na.rm = na.rm,
-      bounds = bounds,
-      orientation = orientation,
-      ...
-    )
-  )
-}
-
-#' @rdname ggplot2-ggproto
+#' @rdname Stat
 #' @format NULL
 #' @usage NULL
 #' @export
-StatDensity <- ggproto("StatDensity", Stat,
+StatDensity <- ggproto(
+  "StatDensity", Stat,
   required_aes = "x|y",
 
   default_aes = aes(x = after_stat(density), y = after_stat(density), fill = NA, weight = NULL),
@@ -102,28 +35,68 @@ StatDensity <- ggproto("StatDensity", Stat,
     }
 
     density <- compute_density(data$x, data$weight, from = range[1],
-      to = range[2], bw = bw, adjust = adjust, kernel = kernel, n = n,
-      bounds = bounds)
+                               to = range[2], bw = bw, adjust = adjust, kernel = kernel, n = n,
+                               bounds = bounds)
     density$flipped_aes <- flipped_aes
     flip_data(density, flipped_aes)
   }
+)
 
+#' @param bw The smoothing bandwidth to be used.
+#'   If numeric, the standard deviation of the smoothing kernel.
+#'   If character, a rule to choose the bandwidth, as listed in
+#'   [stats::bw.nrd()]. Note that automatic calculation of the bandwidth does
+#'   not take weights into account.
+#' @param adjust A multiplicate bandwidth adjustment. This makes it possible
+#'    to adjust the bandwidth while still using the a bandwidth estimator.
+#'    For example, `adjust = 1/2` means use half of the default bandwidth.
+#' @param kernel Kernel. See list of available kernels in [density()].
+#' @param n number of equally spaced points at which the density is to be
+#'   estimated, should be a power of two, see [density()] for
+#'   details
+#' @param trim If `FALSE`, the default, each density is computed on the
+#'   full range of the data. If `TRUE`, each density is computed over the
+#'   range of that group: this typically means the estimated x values will
+#'   not line-up, and hence you won't be able to stack density values.
+#'   This parameter only matters if you are displaying multiple densities in
+#'   one plot or if you are manually adjusting the scale limits.
+#' @param bounds Known lower and upper bounds for estimated data. Default
+#'   `c(-Inf, Inf)` means that there are no (finite) bounds. If any bound is
+#'   finite, boundary effect of default density estimation will be corrected by
+#'   reflecting tails outside `bounds` around their closest edge. Data points
+#'   outside of bounds are removed with a warning.
+#' @eval rd_computed_vars(
+#'  density  = "density estimate.",
+#'  count    = "density * number of points - useful for stacked density plots.",
+#'  wdensity = "density * sum of weights. In absence of weights, the same as
+#'  `count`.",
+#'  scaled   = "density estimate, scaled to maximum of 1.",
+#'  n        = "number of points.",
+#'  ndensity = "alias for `scaled`, to mirror the syntax of [`stat_bin()`]."
+#' )
+#' @export
+#' @rdname geom_density
+stat_density <- make_constructor(
+  StatDensity, geom = "area", position = "stack",
+  orientation = NA
 )
 
 compute_density <- function(x, w, from, to, bw = "nrd0", adjust = 1,
                             kernel = "gaussian", n = 512,
                             bounds = c(-Inf, Inf)) {
-  nx <- length(x)
+  nx <- w_sum <- length(x)
   if (is.null(w)) {
     w <- rep(1 / nx, nx)
   } else {
-    w <- w / sum(w)
+    w_sum <- sum(w)
+    w <- w / w_sum
   }
 
   # Adjust data points and weights to all fit inside bounds
   sample_data <- fit_data_to_bounds(bounds, x, w)
   x <- sample_data$x
   w <- sample_data$w
+  w_sum <- sample_data$w_sum * w_sum
   nx <- length(x)
 
   # if less than 2 points return data frame of NAs and a warning
@@ -135,6 +108,7 @@ compute_density <- function(x, w, from, to, bw = "nrd0", adjust = 1,
       scaled = NA_real_,
       ndensity = NA_real_,
       count = NA_real_,
+      wdensity = NA_real_,
       n = NA_integer_,
       .size = 1
     ))
@@ -143,10 +117,23 @@ compute_density <- function(x, w, from, to, bw = "nrd0", adjust = 1,
   bw <- precompute_bw(x, bw)
   # Decide whether to use boundary correction
   if (any(is.finite(bounds))) {
-    dens <- stats::density(x, weights = w, bw = bw, adjust = adjust,
-                           kernel = kernel, n = n)
+    # To prevent discontinuities, we widen the range before calling the
+    # unbounded estimator (#5641).
+    bounds   <- sort(bounds)
+    range    <- range(from, to)
+    width    <- diff(range)
+    range[1] <- range[1] - width * as.numeric(is.finite(bounds[1]))
+    range[2] <- range[2] + width * as.numeric(is.finite(bounds[2]))
+    n <- n * (sum(is.finite(bounds)) + 1)
 
-    dens <- reflect_density(dens = dens, bounds = bounds, from = from, to = to)
+    dens <- stats::density(
+      x, weights = w, bw = bw, adjust = adjust,
+      kernel = kernel, n = n, from = range[1], to = range[2]
+    )
+    dens <- reflect_density(
+      dens = dens, bounds = bounds,
+      from = range[1], to = range[2]
+    )
   } else {
     dens <- stats::density(x, weights = w, bw = bw, adjust = adjust,
                            kernel = kernel, n = n, from = from, to = to)
@@ -158,6 +145,7 @@ compute_density <- function(x, w, from, to, bw = "nrd0", adjust = 1,
     scaled =  dens$y / max(dens$y, na.rm = TRUE),
     ndensity = dens$y / max(dens$y, na.rm = TRUE),
     count =   dens$y * nx,
+    wdensity = dens$y * w_sum,
     n = nx,
     .size = length(dens$x)
   )
@@ -166,7 +154,7 @@ compute_density <- function(x, w, from, to, bw = "nrd0", adjust = 1,
 # Check if all data points are inside bounds. If not, warn and remove them.
 fit_data_to_bounds <- function(bounds, x, w) {
   is_inside_bounds <- (bounds[1] <= x) & (x <= bounds[2])
-
+  w_sum <- 1
   if (!all(is_inside_bounds)) {
     cli::cli_warn("Some data points are outside of `bounds`. Removing them.")
     x <- x[is_inside_bounds]
@@ -177,7 +165,7 @@ fit_data_to_bounds <- function(bounds, x, w) {
     }
   }
 
-  return(list(x = x, w = w))
+  return(list(x = x, w = w, w_sum = w_sum))
 }
 
 # Update density estimation to mitigate boundary effect at known `bounds`:
@@ -220,9 +208,10 @@ reflect_density <- function(dens, bounds, from, to) {
 # Similar to stats::density.default
 # Once R4.3.0 is the lowest supported version, this function can be replaced by
 # using `density(..., warnWbw = FALSE)`.
-precompute_bw = function(x, bw = "nrd0") {
+precompute_bw <- function(x, bw = "nrd0") {
   bw <- bw[1]
   if (is.character(bw)) {
+    bw <- to_lower_ascii(bw)
     bw <- arg_match0(bw, c("nrd0", "nrd", "ucv", "bcv", "sj", "sj-ste", "sj-dpi"))
     bw <- switch(
       to_lower_ascii(bw),
