@@ -16,7 +16,10 @@ test_that("ggsave can create directories", {
 
   p <- ggplot(mpg, aes(displ, hwy)) + geom_point()
 
-  expect_error(ggsave(path, p))
+  expect_snapshot(
+    ggsave(path, p), error = TRUE,
+    transform = function(x) gsub("directory '.*'\\.$", "directory 'PATH'", x)
+  )
   expect_false(dir.exists(dirname(path)))
 
   # 2 messages: 1 for saving and 1 informing about directory creation
@@ -55,6 +58,9 @@ test_that("ggsave uses theme background as image background", {
   img <- xml2::read_xml(path)
   # Find background rect in svg
   bg <- as.character(xml2::xml_find_first(img, xpath = "d1:rect/@style"))
+  if (is.na(bg)) {
+    bg <- as.character(xml2::xml_find_first(img, xpath = "d1:g/d1:rect/@style"))
+  }
   expect_true(grepl("fill: #00CCCC", bg))
 })
 
@@ -70,6 +76,9 @@ test_that("ggsave can handle blank background", {
   ggsave(path, p, device = "svg", width = 5, height = 5)
   img <- xml2::read_xml(path)
   bg <- as.character(xml2::xml_find_first(img, xpath = "d1:rect/@style"))
+  if (is.na(bg)) {
+    bg <- as.character(xml2::xml_find_first(img, xpath = "d1:g/d1:rect/@style"))
+  }
   expect_true(grepl("fill: none", bg))
 })
 
@@ -77,23 +86,20 @@ test_that("ggsave warns about empty or multiple filenames", {
   plot <- ggplot(mtcars, aes(disp, mpg)) + geom_point()
 
   withr::with_tempfile(c("file1", "file2"), fileext = ".png", {
-    expect_warning(
-      suppressMessages(ggsave(c(file1, file2), plot)),
-      "`filename` must have length 1"
+    expect_snapshot(
+      x <- suppressMessages(ggsave(c(file1, file2), plot)),
+      transform = function(x) gsub(" \\'.*\\.png\\'", "'PATH'", x)
     )
   })
 
-  expect_error(
-    ggsave(character(), plot),
-    "`filename` must be a single string"
-  )
+  expect_snapshot(ggsave(character(), plot), error = TRUE)
 })
 
 test_that("ggsave fails informatively for no-extension filenames", {
   plot <- ggplot(mtcars, aes(disp, mpg)) + geom_point()
-  expect_error(
-    ggsave(tempfile(), plot),
-    "Can't save to"
+  expect_snapshot(
+    ggsave(tempfile(), plot), error = TRUE,
+    transform = function(x) gsub("to .*\\.$", "to PATH", x)
   )
 })
 
@@ -112,9 +118,9 @@ test_that("uses 7x7 if no graphics device open", {
 })
 
 test_that("warned about large plot unless limitsize = FALSE", {
-  expect_error(plot_dim(c(50, 50)), "exceed 50 inches")
+  expect_snapshot(plot_dim(c(50, 50)), error = TRUE)
   expect_equal(plot_dim(c(50, 50), limitsize = FALSE), c(50, 50))
-  expect_error(plot_dim(c(15000, 15000), units = "px"), "in pixels).")
+  expect_snapshot(plot_dim(c(15000, 15000), units = "px"), error = TRUE)
 })
 
 test_that("scale multiplies height & width", {
@@ -125,19 +131,19 @@ test_that("scale multiplies height & width", {
 # plot_dev ---------------------------------------------------------------------
 
 test_that("unknown device triggers error", {
-  expect_snapshot_error(plot_dev(1))
-  expect_error(plot_dev("xyz"), "Unknown graphics device")
-  expect_error(plot_dev(NULL, "test.xyz"), "Unknown graphics device")
+  expect_snapshot_error(validate_device(1))
+  expect_snapshot(validate_device("xyz"), error = TRUE)
+  expect_snapshot(validate_device(NULL, "test.xyz"), error = TRUE)
 })
 
 
 test_that("text converted to function", {
-  expect_identical(body(plot_dev("png"))[[1]], quote(png_dev))
-  expect_identical(body(plot_dev("pdf"))[[1]], quote(grDevices::pdf))
+  expect_identical(body(validate_device("png"))[[1]], quote(png_dev))
+  expect_identical(body(validate_device("pdf"))[[1]], quote(grDevices::pdf))
 })
 
 test_that("if device is NULL, guess from extension", {
-  expect_identical(body(plot_dev(NULL, "test.png"))[[1]], quote(png_dev))
+  expect_identical(body(validate_device(NULL, "test.png"))[[1]], quote(png_dev))
 })
 
 # parse_dpi ---------------------------------------------------------------
