@@ -1,6 +1,5 @@
 test_that("scale_hue() checks the type input", {
-  pal <- pal_qualitative(type = 1:4)
-  expect_snapshot_error(pal(4))
+  expect_snapshot_error(pal_qualitative(type = 1:4))
   pal <- pal_qualitative(type = colors())
   expect_silent(pal(4))
   pal <- pal_qualitative(type = list(colors()[1:10], colors()[11:30]))
