@@ -1,4 +1,19 @@
 
+test_that("date(time) scales coerce data types", {
+
+  date     <- as.Date("2024-11-11")
+  datetime <- as.POSIXct(date)
+
+  sc <- scale_x_datetime()
+  df <- sc$transform_df(data_frame0(x = date))
+  expect_equal(df$x, as.numeric(datetime))
+
+  sc <- scale_x_date()
+  df <- sc$transform_df(data_frame0(x = datetime))
+  expect_equal(df$x, as.numeric(date))
+
+})
+
 # Visual tests ------------------------------------------------------------
 
 test_that("date scale draws correctly", {
