@@ -2,10 +2,10 @@ test_that("building a plot does not affect its scales", {
   dat <- data_frame(x = rnorm(20), y = rnorm(20))
 
   p <- ggplot(dat, aes(x, y)) + geom_point()
-  expect_equal(length(p$scales$scales), 0)
+  expect_length(p@scales$scales, 0)
 
   ggplot_build(p)
-  expect_equal(length(p$scales$scales), 0)
+  expect_length(p@scales$scales, 0)
 })
 
 test_that("ranges update only for variables listed in aesthetics", {
@@ -52,7 +52,7 @@ test_that("identity scale preserves input values", {
     scale_shape_identity() +
     scale_size_identity() +
     scale_alpha_identity()
-  d1 <- layer_data(p1)
+  d1 <- get_layer_data(p1)
 
   expect_equal(d1$colour, as.character(df$z))
   expect_equal(d1$fill, as.character(df$z))
@@ -66,7 +66,7 @@ test_that("identity scale preserves input values", {
     geom_point() +
     scale_discrete_identity(aesthetics = c("colour", "fill", "shape")) +
     scale_continuous_identity(aesthetics = c("size", "alpha"))
-  d2 <- layer_data(p2)
+  d2 <- get_layer_data(p2)
 
   expect_equal(d1, d2)
 })
@@ -82,7 +82,7 @@ test_that("position scales are updated by all position aesthetics", {
   )
 
   base <- ggplot(df, aes(x = 1, y = 1)) + geom_point()
-  plots <- lapply(aesthetics, function(x) base %+% x)
+  plots <- lapply(aesthetics, ggplot_add, plot = base)
   ranges <- lapply(plots, pranges)
 
   lapply(ranges, function(range) {
@@ -115,10 +115,12 @@ test_that("oob affects position values", {
   mid_censor <- cdata(base + y_scale(c(3, 7), censor))
   handle <- GeomBar$handle_na
 
-  expect_warning(low_censor[[1]] <- handle(low_censor[[1]], list(na.rm = FALSE)),
-    "Removed 1 row containing missing values or values outside the scale range")
-  expect_warning(mid_censor[[1]] <- handle(mid_censor[[1]], list(na.rm = FALSE)),
-    "Removed 3 rows containing missing values or values outside the scale range")
+  expect_snapshot_warning(
+    low_censor[[1]] <- handle(low_censor[[1]], list(na.rm = FALSE)),
+  )
+  expect_snapshot_warning(
+    mid_censor[[1]] <- handle(mid_censor[[1]], list(na.rm = FALSE)),
+  )
 
   low_squish <- cdata(base + y_scale(c(0, 5), squish))
   mid_squish <- cdata(base + y_scale(c(3, 7), squish))
@@ -144,18 +146,18 @@ test_that("all-Inf layers are not used for determining the type of scale", {
     geom_point()
 
   b1 <- ggplot_build(p1)
-  expect_s3_class(b1$layout$panel_scales_x[[1]], "ScaleDiscretePosition")
+  expect_s3_class(b1@layout$panel_scales_x[[1]], "ScaleDiscretePosition")
 
   p2 <- ggplot() +
     # If the layer non-Inf value, it's considered
     annotate("rect", xmin = -Inf, xmax = 0, ymin = -Inf, ymax = Inf, fill = "black")
 
   b2 <- ggplot_build(p2)
-  expect_s3_class(b2$layout$panel_scales_x[[1]], "ScaleContinuousPosition")
+  expect_s3_class(b2@layout$panel_scales_x[[1]], "ScaleContinuousPosition")
 })
 
 test_that("scales are looked for in appropriate place", {
-  xlabel <- function(x) ggplot_build(x)$layout$panel_scales_x[[1]]$name
+  xlabel <- function(x) ggplot_build(x)@layout$panel_scales_x[[1]]$name
   p0 <- ggplot(mtcars, aes(mpg, wt)) + geom_point() + scale_x_continuous("0")
   expect_equal(xlabel(p0), "0")
 
@@ -201,7 +203,7 @@ test_that("scales warn when transforms introduces non-finite values", {
     geom_point(size = 5) +
     scale_y_log10()
 
-  expect_warning(ggplot_build(p), "log-10 transformation introduced infinite values.")
+  expect_snapshot_warning(ggplot_build(p))
 })
 
 test_that("size and alpha scales throw appropriate warnings for factors", {
@@ -214,21 +216,18 @@ test_that("size and alpha scales throw appropriate warnings for factors", {
   p <- ggplot(df, aes(x, y))
 
   # There should be warnings when unordered factors are mapped to size/alpha
-  expect_warning(
-    ggplot_build(p + geom_point(aes(size = d))),
-    "Using size for a discrete variable is not advised."
+  expect_snapshot_warning(
+    ggplot_build(p + geom_point(aes(size = d)))
   )
-  expect_warning(
-    ggplot_build(p + geom_point(aes(alpha = d))),
-    "Using alpha for a discrete variable is not advised."
+  expect_snapshot_warning(
+    ggplot_build(p + geom_point(aes(alpha = d)))
   )
-  expect_warning(
-    ggplot_build(p + geom_line(aes(linewidth = d, group = 1))),
-    "Using linewidth for a discrete variable is not advised."
+  expect_snapshot_warning(
+    ggplot_build(p + geom_line(aes(linewidth = d, group = 1)))
   )
   # There should be no warnings for ordered factors
-  expect_warning(ggplot_build(p + geom_point(aes(size = o))), NA)
-  expect_warning(ggplot_build(p + geom_point(aes(alpha = o))), NA)
+  expect_no_warning(ggplot_build(p + geom_point(aes(size = o))))
+  expect_no_warning(ggplot_build(p + geom_point(aes(alpha = o))))
 })
 
 test_that("shape scale throws appropriate warnings for factors", {
@@ -241,12 +240,11 @@ test_that("shape scale throws appropriate warnings for factors", {
   p <- ggplot(df, aes(x, y))
 
   # There should be no warnings when unordered factors are mapped to shape
-  expect_warning(ggplot_build(p + geom_point(aes(shape = d))), NA)
+  expect_no_warning(ggplot_build(p + geom_point(aes(shape = d))))
 
   # There should be warnings for ordered factors
-  expect_warning(
-    ggplot_build(p + geom_point(aes(shape = o))),
-    "Using shapes for an ordinal variable is not advised"
+  expect_snapshot_warning(
+    ggplot_build(p + geom_point(aes(shape = o)))
   )
 })
 
@@ -258,7 +256,7 @@ test_that("aesthetics can be set independently of scale name", {
   p <- ggplot(df, aes(x, y, fill = y)) +
     scale_colour_manual(values = c("red", "green", "blue"), aesthetics = "fill")
 
-  expect_equal(layer_data(p)$fill, c("red", "green", "blue"))
+  expect_equal(get_layer_data(p)$fill, c("red", "green", "blue"))
 })
 
 test_that("multiple aesthetics can be set with one function call", {
@@ -272,8 +270,8 @@ test_that("multiple aesthetics can be set with one function call", {
       aesthetics = c("colour", "fill")
     )
 
-  expect_equal(layer_data(p)$colour, c("grey20", "grey40", "grey60"))
-  expect_equal(layer_data(p)$fill, c("red", "green", "blue"))
+  expect_equal(get_layer_data(p)$colour, c("grey20", "grey40", "grey60"))
+  expect_equal(get_layer_data(p)$fill, c("red", "green", "blue"))
 
   # color order is determined by data order, and breaks are combined where possible
   df <- data_frame(
@@ -286,8 +284,8 @@ test_that("multiple aesthetics can be set with one function call", {
       aesthetics = c("fill", "colour")
     )
 
-  expect_equal(layer_data(p)$colour, c("cyan", "red", "green"))
-  expect_equal(layer_data(p)$fill, c("red", "green", "blue"))
+  expect_equal(get_layer_data(p)$colour, c("cyan", "red", "green"))
+  expect_equal(get_layer_data(p)$fill, c("red", "green", "blue"))
 })
 
 test_that("limits with NA are replaced with the min/max of the data for continuous scales", {
@@ -345,12 +343,12 @@ test_that("scale_apply preserves class and attributes", {
 
   # Perform identity transformation via `scale_apply`
   out <- with_bindings(scale_apply(
-    df, "x", "transform", 1:2, plot$layout$panel_scales_x
+    df, "x", "transform", 1:2, plot@layout$panel_scales_x
   )[[1]], `c.baz` = `c.baz`, `[.baz` = `[.baz`, .env = global_env())
 
   # Check that it errors on bad scale ids
   expect_snapshot_error(scale_apply(
-    df, "x", "transform", c(NA, 1), plot$layout$panel_scales_x
+    df, "x", "transform", c(NA, 1), plot@layout$panel_scales_x
   ))
 
   # Check class preservation
@@ -364,7 +362,7 @@ test_that("scale_apply preserves class and attributes", {
   class(df$x) <- "foobar"
 
   out <- with_bindings(scale_apply(
-    df, "x", "transform", 1:2, plot$layout$panel_scales_x
+    df, "x", "transform", 1:2, plot@layout$panel_scales_x
   )[[1]], `c.baz` = `c.baz`, `[.baz` = `[.baz`, .env = global_env())
 
   expect_false(inherits(out, "foobar"))
@@ -432,21 +430,18 @@ test_that("scales accept lambda notation for function input", {
 
 test_that("breaks and labels are correctly checked", {
   expect_snapshot_error(check_breaks_labels(1:10, letters))
-  p <- ggplot(mtcars) + geom_point(aes(mpg, disp)) + scale_x_continuous(breaks = NA)
-  expect_snapshot_error(ggplot_build(p))
+  expect_snapshot_error(scale_x_continuous(breaks = NA))
   p <- ggplot(mtcars) + geom_point(aes(mpg, disp)) + scale_x_continuous(minor_breaks = NA)
   expect_snapshot_error(ggplot_build(p))
   p <- ggplot(mtcars) + geom_point(aes(mpg, disp)) + scale_x_continuous(labels = NA)
   expect_snapshot_error(ggplotGrob(p))
   p <- ggplot(mtcars) + geom_point(aes(mpg, disp)) + scale_x_continuous(labels = function(x) 1:2)
   expect_snapshot_error(ggplotGrob(p))
-  p <- ggplot(mtcars) + geom_bar(aes(factor(gear))) + scale_x_discrete(breaks = NA)
-  expect_snapshot_error(ggplot_build(p))
+  expect_snapshot_error(scale_x_discrete(breaks = NA))
   p <- ggplot(mtcars) + geom_bar(aes(factor(gear))) + scale_x_discrete(labels = NA)
   expect_snapshot_error(ggplotGrob(p))
 
-  p <- ggplot(mtcars) + geom_bar(aes(mpg)) + scale_x_binned(breaks = NA)
-  expect_snapshot_error(ggplot_build(p))
+  expect_snapshot_error(scale_x_binned(breaks = NA))
   p <- ggplot(mtcars) + geom_bar(aes(mpg)) + scale_x_binned(labels = NA)
   expect_snapshot_error(ggplotGrob(p))
   p <- ggplot(mtcars) + geom_bar(aes(mpg)) + scale_x_binned(labels = function(x) 1:2)
@@ -459,7 +454,7 @@ test_that("staged aesthetics are backtransformed properly (#4155)", {
     scale_x_sqrt(limits = c(0, 16), breaks = c(2, 4, 8))
 
   # x / 2 should be 16 / 2 = 8, thus the result should be sqrt(8) on scale_x_sqrt()
-  expect_equal(layer_data(p)$x, sqrt(8))
+  expect_equal(get_layer_data(p)$x, sqrt(8))
 })
 
 test_that("numeric scale transforms can produce breaks", {
@@ -471,59 +466,24 @@ test_that("numeric scale transforms can produce breaks", {
     scale$get_transformation()$inverse(view$get_breaks())
   }
 
-  expect_equal(test_breaks("asn", limits = c(0, 1)),
-               seq(0, 1, by = 0.25))
-
-  expect_equal(test_breaks("sqrt", limits = c(0, 10)),
-               seq(0, 10, by = 2.5))
-
-  expect_equal(test_breaks("atanh", limits = c(-0.9, 0.9)),
-               c(NA, -0.5, 0, 0.5, NA))
-
-  expect_equal(test_breaks(transform_boxcox(0), limits = c(1, 10)),
-               c(NA, 2.5, 5.0, 7.5, 10))
-
-  expect_equal(test_breaks(transform_modulus(0), c(-10, 10)),
-               seq(-10, 10, by = 5))
-
-  expect_equal(test_breaks(transform_yj(0), c(-10, 10)),
-               seq(-10, 10, by = 5))
-
-  expect_equal(test_breaks("exp", c(-10, 10)),
-               seq(-10, 10, by = 5))
-
-  expect_equal(test_breaks("identity", limits = c(-10, 10)),
-               seq(-10, 10, by = 5))
-
-  # irrational numbers, so snapshot values
+  expect_snapshot(test_breaks("asn", limits = c(0, 1)))
+  expect_snapshot(test_breaks("sqrt", limits = c(0, 10)))
+  expect_snapshot(test_breaks("atanh", limits = c(-0.9, 0.9)))
+  expect_snapshot(test_breaks(transform_boxcox(0), limits = c(1, 10)))
+  expect_snapshot(test_breaks(transform_modulus(0), c(-10, 10)))
+  expect_snapshot(test_breaks(transform_yj(0), c(-10, 10)))
+  expect_snapshot(test_breaks("exp", c(-10, 10)))
+  expect_snapshot(test_breaks("identity", limits = c(-10, 10)))
   expect_snapshot(test_breaks("log", limits = c(0.1, 1000)))
-
-  expect_equal(test_breaks("log10", limits = c(0.1, 1000)),
-               10 ^ seq(-1, 3))
-
-  expect_equal(test_breaks("log2", limits = c(0.5, 32)),
-               c(0.5, 2, 8, 32))
-
-  expect_equal(test_breaks("log1p", limits = c(0, 10)),
-               seq(0, 10, by = 2.5))
-
-  expect_equal(test_breaks("pseudo_log", limits = c(-10, 10)),
-               seq(-10, 10, by = 5))
-
-  expect_equal(test_breaks("logit", limits = c(0.001, 0.999)),
-               c(NA, 0.25, 0.5, 0.75, NA))
-
-  expect_equal(test_breaks("probit", limits = c(0.001, 0.999)),
-               c(NA, 0.25, 0.5, 0.75, NA))
-
-  expect_equal(test_breaks("reciprocal", limits = c(1, 10)),
-               c(NA, 2.5, 5, 7.5, 10))
-
-  expect_equal(test_breaks("reverse", limits = c(-10, 10)),
-               seq(-10, 10, by = 5))
-
-  expect_equal(test_breaks("sqrt", limits = c(0, 10)),
-               seq(0, 10, by = 2.5))
+  expect_snapshot(test_breaks("log10", limits = c(0.1, 1000)))
+  expect_snapshot(test_breaks("log2", limits = c(0.5, 32)))
+  expect_snapshot(test_breaks("log1p", limits = c(0, 10)))
+  expect_snapshot(test_breaks("pseudo_log", limits = c(-10, 10)))
+  expect_snapshot(test_breaks("logit", limits = c(0.001, 0.999)))
+  expect_snapshot(test_breaks("probit", limits = c(0.001, 0.999)))
+  expect_snapshot(test_breaks("reciprocal", limits = c(1, 10)))
+  expect_snapshot(test_breaks("reverse", limits = c(-10, 10)))
+  expect_snapshot(test_breaks("sqrt", limits = c(0, 10)))
 })
 
 test_that("scale functions accurately report their calls", {
@@ -730,3 +690,112 @@ test_that("Discrete scales with only NAs return `na.value`", {
   sc$train(x)
   expect_equal(sc$map(x), c(NA_real_, NA_real_))
 })
+
+test_that("continuous scales warn about faulty `limits`", {
+  expect_snapshot(scale_x_continuous(limits = c("A", "B")), error = TRUE)
+  expect_snapshot(scale_x_continuous(limits = 1:3), error = TRUE)
+})
+
+test_that("populating palettes works", {
+
+  scl <- scales_list()
+  scl$add(scale_colour_discrete(aesthetics = c("colour", "fill")))
+
+  my_theme <- theme(
+    palette.colour.discrete = c("white", "black"),
+    palette.fill.discrete = c("red", "blue")
+  )
+
+  scl$set_palettes(my_theme)
+  expect_equal(scl$scales[[1]]$palette(2), c("white", "black"))
+
+  # Scales with >1 aesthetic
+  scl <- scales_list()
+  scl$add(scale_colour_discrete(aesthetics = c("colour", "fill")))
+
+  my_theme$palette.colour.discrete <- NULL
+
+  scl$set_palettes(my_theme)
+  expect_equal(scl$scales[[1]]$palette(2), c("red", "blue"))
+
+})
+
+test_that("discrete scales work with NAs in arbitrary positions", {
+  # Prevents intermediate caching of palettes
+  map <- function(x, limits) {
+    sc <- scale_colour_manual(
+      values = c("red", "green", "blue"),
+      na.value = "gray"
+    )
+    sc$map(x, limits)
+  }
+
+  # All inputs should yield output regardless of where NA is
+  input  <- c("A", "B", "C", NA)
+  output <- c("red", "green", "blue", "gray")
+
+  test <- map(input, limits = c("A", "B", "C", NA))
+  expect_equal(test, output)
+
+  test <- map(input, limits = c("A", NA, "B", "C"))
+  expect_equal(test, output)
+
+  test <- map(input, limits = c(NA, "A", "B", "C"))
+  expect_equal(test, output)
+
+})
+
+test_that("ViewScales can make fixed copies", {
+
+  p1 <- ggplot(mpg, aes(drv, displ)) +
+    geom_boxplot() +
+    annotate("point", x = 5, y = 10) +
+    scale_x_discrete(labels = c("four-wheel", "forward", "reverse"))
+
+  b1 <- ggplot_build(p1)@layout$panel_params[[1]]
+
+  # We build a second plot with the first plot's scales
+  p2 <- ggplot(mpg, aes(drv, cyl)) +
+    geom_violin() +
+    annotate("point", x = 15, y = 100) +
+    b1$x$make_fixed_copy() +
+    b1$y$make_fixed_copy()
+  b2 <- ggplot_build(p2)
+
+  # Breaks and labels should respect p1's limits
+  x <- get_guide_data(b2, "x")
+  expect_equal(x$x, 0.6:2.6 / diff(b1$x.range))
+  expect_equal(x$.label, c("four-wheel", "forward", "reverse"))
+
+  y <- get_guide_data(b2, "y")
+  expect_equal(y$y, rescale(seq(2.5, 10, by = 2.5), from = b1$y.range))
+})
+
+test_that("discrete scales can map to 2D structures", {
+
+  p <- ggplot(mtcars, aes(disp, mpg, colour = factor(cyl))) +
+    geom_point()
+
+  # Test it can map to a vctrs rcrd class
+  rcrd <- new_rcrd(list(a = LETTERS[1:3], b = 3:1))
+
+  ld <- layer_data(p + scale_colour_manual(values = rcrd, na.value = NA))
+  expect_s3_class(ld$colour, "vctrs_rcrd")
+  expect_length(ld$colour, nrow(mtcars))
+
+  # Test it can map to data.frames
+  df <- data_frame0(a = LETTERS[1:3], b = 3:1)
+  my_pal <- function(n) vec_slice(df, seq_len(n))
+
+  ld <- layer_data(p + discrete_scale("colour", palette = my_pal))
+  expect_s3_class(ld$colour, "data.frame")
+  expect_equal(dim(ld$colour), c(nrow(mtcars), ncol(df)))
+
+  # Test it can map to matrices
+  mtx <- cbind(a = LETTERS[1:3], b = LETTERS[4:6])
+  my_pal <- function(n) vec_slice(mtx, seq_len(n))
+
+  ld <- layer_data(p + discrete_scale("colour", palette = my_pal))
+  expect_true(is.matrix(ld$colour))
+  expect_equal(dim(ld$colour), c(nrow(mtcars), ncol(df)))
+})
