@@ -25,7 +25,7 @@ test_that("dup_axis() works", {
       name = "Unit A",
       sec.axis = dup_axis()
     )
-  scale <- layer_scales(p)$x
+  scale <- get_panel_scales(p)$x
   expect_equal(scale$sec_name(), scale$name)
   breaks <- scale$break_info()
   expect_equal(breaks$minor_source, breaks$sec.minor_source_user)
@@ -45,7 +45,7 @@ test_that("sec_axis() works with subtraction", {
     scale_y_continuous(
       sec.axis = sec_axis(~1-.)
     )
-  scale <- layer_scales(p)$y
+  scale <- get_panel_scales(p)$y
   expect_equal(scale$sec_name(), scale$name)
   breaks <- scale$break_info()
   expect_equal(breaks$minor_source, breaks$sec.minor_source_user)
@@ -81,7 +81,7 @@ test_that("sec_axis() breaks work for log-transformed scales", {
     geom_point() +
     scale_y_log10(sec.axis = dup_axis())
 
-  scale <- layer_scales(p)$y
+  scale <- get_panel_scales(p)$y
   breaks <- scale$break_info()
 
   # test value
@@ -96,7 +96,7 @@ test_that("sec_axis() breaks work for log-transformed scales", {
     geom_point() +
     scale_y_log10(sec.axis = sec_axis(~ . * 100))
 
-  scale <- layer_scales(p)$y
+  scale <- get_panel_scales(p)$y
   breaks <- scale$break_info()
 
   # test value
@@ -113,7 +113,7 @@ test_that("sec_axis() breaks work for log-transformed scales", {
     geom_point() +
     scale_y_log10(breaks = custom_breaks, sec.axis = sec_axis(~ . * 100))
 
-  scale <- layer_scales(p)$y
+  scale <- get_panel_scales(p)$y
   breaks <- scale$break_info()
 
   expect_equal(breaks$major_source, log(custom_breaks, base = 10))
@@ -131,7 +131,7 @@ test_that("custom breaks work", {
         breaks = custom_breaks
       )
     )
-  scale <- layer_scales(p)$x
+  scale <- get_panel_scales(p)$x
   breaks <- scale$break_info()
   expect_equal(custom_breaks, breaks$sec.major_source_user)
 })
@@ -171,11 +171,11 @@ test_that("sec axis works with tidy eval", {
     g
   }
 
-  t <- tibble(x = letters, y = seq(10, 260, 10), z = 1:26)
+  t <- data_frame0(x = letters, y = seq(10, 260, 10), z = 1:26)
 
   p <- f(t, x, y, z)
 
-  scale <- layer_scales(p)$y
+  scale <- get_panel_scales(p)$y
   breaks <- scale$break_info()
 
   # test transform
@@ -194,7 +194,7 @@ test_that("sec_axis() handles secondary power transformations", {
     geom_point() +
     scale_y_continuous(sec.axis = sec_axis(transform = (~ 2^.)))
 
-  scale <- layer_scales(p)$y
+  scale <- get_panel_scales(p)$y
   breaks <- scale$break_info()
 
   expect_equal(round(breaks$major[4:6], 2), round(breaks$sec.major[c(1, 2, 4)], 2))
@@ -242,7 +242,7 @@ test_that("sec_axis() respects custom transformations", {
   expect_doppelganger(
     "sec_axis, custom transform",
     ggplot(dat, aes(x = x, y = y)) +
-      geom_line(linewidth = 1, na.rm = T) +
+      geom_line(linewidth = 1, na.rm = TRUE) +
       scale_y_continuous(
         transform =
           magnify_trans_log(interval_low = 0.5, interval_high = 1, reducer = 0.5, reducer2 = 8), breaks =
@@ -274,7 +274,7 @@ test_that("sec_axis works with date/time/datetime scales", {
   dt <- ggplot(df, aes(dx, price)) +
     geom_line() +
     scale_x_datetime(sec.axis = dup_axis())
-  scale <- layer_scales(dt)$x
+  scale <- get_panel_scales(dt)$x
   breaks <- scale$break_info()
   expect_equal(breaks$major_source, breaks$sec.major_source_user)
 
@@ -282,7 +282,7 @@ test_that("sec_axis works with date/time/datetime scales", {
   dt <- ggplot(df, aes(date, price)) +
     geom_line() +
     scale_x_date(sec.axis = dup_axis())
-  scale <- layer_scales(dt)$x
+  scale <- get_panel_scales(dt)$x
   breaks <- scale$break_info()
   expect_equal(breaks$major_source, breaks$sec.major_source_user)
 
@@ -295,7 +295,7 @@ test_that("sec_axis works with date/time/datetime scales", {
         name = "UTC+12"
       )
     )
-  scale <- layer_scales(dt)$x
+  scale <- get_panel_scales(dt)$x
   breaks <- scale$break_info()
 
   expect_equal(
@@ -362,21 +362,60 @@ test_that("sec_axis() works for power transformations (monotonicity test doesn't
   p <- ggplot(data = testdat, aes(x = x, y = y)) +
     geom_point() +
     scale_y_continuous(sec.axis = sec_axis(transform = ~ .^0.5))
-  scale <- layer_scales(p)$y
+  scale <- get_panel_scales(p)$y
   breaks <- scale$break_info()
-  expect_equal(breaks$major, sqrt(breaks$sec.major), tolerance = .005)
+  expect_equal(breaks$major, sqrt(breaks$sec.major), tolerance = 0.005)
 
   p <- ggplot(foo, aes(x, y)) +
     geom_point() +
     scale_x_sqrt(sec.axis = dup_axis())
-  scale <- layer_scales(p)$x
+  scale <- get_panel_scales(p)$x
   breaks <- scale$break_info()
-  expect_equal(breaks$major, breaks$sec.major, tolerance = .001)
+  expect_equal(breaks$major, breaks$sec.major, tolerance = 0.001)
 
   p <- ggplot(foo, aes(x, y)) +
     geom_point() +
     scale_x_sqrt(sec.axis = sec_axis(~ . * 100))
-  scale <- layer_scales(p)$x
+  scale <- get_panel_scales(p)$x
   breaks <- scale$break_info()
-  expect_equal(breaks$major, breaks$sec.major, tolerance = .001)
+  expect_equal(breaks$major, breaks$sec.major, tolerance = 0.001)
+})
+
+test_that("discrete scales can have secondary axes", {
+
+  data <- data.frame(x = c("A", "B", "C"), y = c("D", "E", "F"))
+  p <- ggplot(data, aes(x, y)) +
+    geom_point() +
+    scale_x_discrete(sec.axis = dup_axis(labels = c("foo", "bar", "baz"))) +
+    scale_y_discrete(sec.axis = dup_axis(
+      breaks = c(1.5, 2.5), labels = c("grault", "garply")
+    ))
+  b <- ggplot_build(p)
+
+  x <- get_guide_data(b, "x.sec")
+  expect_equal(x$.value, 1:3, ignore_attr = TRUE)
+  expect_equal(x$.label, c("foo", "bar", "baz"))
+
+  y <- get_guide_data(b, "y.sec")
+  expect_equal(y$.value, c(1.5, 2.5), ignore_attr = TRUE)
+  expect_equal(y$.label, c("grault", "garply"))
+})
+
+test_that("n.breaks is respected by secondary axes (#4483)", {
+
+  b <- ggplot_build(
+    ggplot(data.frame(x = c(0, 10)), aes(x, x)) +
+      scale_y_continuous(
+        n.breaks = 11,
+        sec.axis = sec_axis(~.x*100)
+      )
+  )
+
+  # We get scale breaks via guide data
+  prim <- get_guide_data(b, "y")
+  sec  <- get_guide_data(b, "y.sec")
+
+  expect_equal(prim$.value, sec$.value) # .value is in primary scale
+  expect_equal(prim$.label, as.character(seq(0, 10, length.out = 11)))
+  expect_equal(sec$.label,  as.character(seq(0, 1000, length.out = 11)))
 })
