@@ -2,14 +2,14 @@ test_that("finite_cases.data.frame", {
   finite_cases <- function(x) cases(x, is_finite)
 
   # All finite --------------------------------------------------------------
-  expect_identical(finite_cases(data_frame(x = 4)),              TRUE)          # 1x1
-  expect_identical(finite_cases(data_frame(x = 4, y = 11)),      TRUE)          # 1x2
+  expect_true(finite_cases(data_frame(x = 4)))          # 1x1
+  expect_true(finite_cases(data_frame(x = 4, y = 11)))          # 1x2
   expect_identical(finite_cases(data_frame(x = 4:5)),            c(TRUE, TRUE)) # 2x1
   expect_identical(finite_cases(data_frame(x = 4:5, y = 11:12)), c(TRUE, TRUE)) # 2x2
 
   # Has one NA --------------------------------------------------------------
-  expect_identical(finite_cases(data_frame(x = NA)),                      FALSE)           # 1x1
-  expect_identical(finite_cases(data_frame(x = 4, y = NA)),               FALSE)           # 1x2
+  expect_false(finite_cases(data_frame(x = NA)))           # 1x1
+  expect_false(finite_cases(data_frame(x = 4, y = NA)))           # 1x2
   expect_identical(finite_cases(data_frame(x = c(4, NA))),                c(TRUE,  FALSE)) # 2x1
   expect_identical(finite_cases(data_frame(x = c(4, NA), y = c(11, NA))), c(TRUE,  FALSE)) # 2x2
   expect_identical(finite_cases(data_frame(x = c(4, NA), y = c(NA, 12))), c(FALSE, FALSE)) # 2x2
@@ -17,7 +17,7 @@ test_that("finite_cases.data.frame", {
 
   # Testing NaN and Inf, using miscellaneous data shapes --------------------
   expect_identical(finite_cases(data_frame(x = c(4, NaN))),                c(TRUE, FALSE))
-  expect_identical(finite_cases(data_frame(x = Inf)),                      FALSE)
+  expect_false(finite_cases(data_frame(x = Inf)))
   expect_identical(finite_cases(data_frame(x = c(4, 5), y = c(-Inf, 12))), c(FALSE, TRUE))
 })
 
@@ -87,7 +87,7 @@ test_that("parse_safe works with multi expressions", {
 })
 
 test_that("x and y aesthetics have the same length", {
-  expect_equal(length(ggplot_global$x_aes), length(ggplot_global$y_aes))
+  expect_length(ggplot_global$x_aes, length(ggplot_global$y_aes))
 })
 
 test_that("check_required_aesthetics() errors on missing", {
@@ -106,9 +106,7 @@ test_that("remove_missing checks input", {
 
 test_that("characters survive remove_missing", {
   data <- data_frame0(x = c("A", NA))
-  expect_warning(
-    new <- remove_missing(data, finite = TRUE)
-  )
+  expect_snapshot_warning(new <- remove_missing(data, finite = TRUE))
   expect_equal(new, data_frame0(x = "A"))
 })
 
@@ -132,10 +130,6 @@ test_that("cut_*() checks its input and output", {
   expect_snapshot_error(cut_width(1:10, 1, center = 0, boundary = 0.5))
 })
 
-test_that("interleave() checks the vector lengths", {
-  expect_snapshot_error(interleave(1:4, numeric()))
-})
-
 test_that("vec_rbind0 can combined ordered factors", {
 
   withr::local_options(lifecycle_verbosity = "warning")
@@ -144,20 +138,17 @@ test_that("vec_rbind0 can combined ordered factors", {
   # However, it was technically challenging to reduce the numbers of warnings
   # See #5139 for more details
 
-  expect_warning(
-    expect_warning(
-      expect_warning(
+  lifecycle::expect_deprecated(
+    lifecycle::expect_deprecated(
+      lifecycle::expect_deprecated(
         {
           test <- vec_rbind0(
             data_frame0(a = factor(c("A", "B"), ordered = TRUE)),
             data_frame0(a = factor(c("B", "C"), ordered = TRUE))
           )
-        },
-        "<ordered> and <ordered>", class = "lifecycle_warning_deprecated"
-      ),
-      "<ordered> and <factor>", class = "lifecycle_warning_deprecated"
-    ),
-    "<ordered> and <factor>", class = "lifecycle_warning_deprecated"
+        }
+      )
+    )
   )
 
   # Should be <factor> not <ordered/factor>, hence the 'exact'
@@ -189,10 +180,41 @@ test_that("expose/ignore_data() can round-trip a data.frame", {
   # data.frame with ignored columns
   df <- data_frame0(a = 1:3, b = I(4:6), c = LETTERS[1:3], d = I(LETTERS[4:6]))
   test <- .ignore_data(df)[[1]]
-  expect_equal(names(test), c("a", "c", ".ignored"))
-  expect_equal(names(test$.ignored), c("b", "d"))
+  expect_named(test, c("a", "c", ".ignored"))
+  expect_named(test$.ignored, c("b", "d"))
 
   test <- .expose_data(test)[[1]]
   expect_equal(test, df[, c("a", "c", "b", "d")])
 
 })
+
+test_that("allow_lambda converts the correct cases", {
+
+  f <- allow_lambda(function(x) x + 1)
+  expect_equal(f(1), 2)
+
+  f <- allow_lambda(~ .x + 1)
+  expect_equal(f(1), 2)
+
+  f <- allow_lambda("A")
+  expect_equal(f, "A")
+
+  f <- allow_lambda(expression(A))
+  expect_equal(f, expression(A))
+
+  f <- allow_lambda(bquote("foo"~"bar"))
+  expect_equal(f, call("~", "foo", "bar"))
+})
+
+test_that("summary method gives a nice summary", {
+  # This test isn't important enough to break anything on CRAN
+  skip_on_cran()
+
+  p <- ggplot(mpg, aes(displ, hwy, colour = drv)) +
+    geom_point() +
+    scale_x_continuous() +
+    scale_colour_brewer() +
+    facet_grid(year ~ cyl)
+
+  expect_snapshot(summary(p))
+})
